package yeoupooh.scraper;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.StringTokenizer;

/**
 * @author pooh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class Scraper
{
    private final String ENGINE_VERSION = "1.43";
    private final String SCRIPT_VERSION = "1.4";
    private final String TAIL_MESSAGE =
        "Powered by Scraper (E:v"
            + ENGINE_VERSION
            + ", S:v"
            + SCRIPT_VERSION
            + ")";
    private ILogger m_logger = null;

    public Scraper()
    {
        m_logger = new DefaultLogger();
    }

    public void setLogger(ILogger logger)
    {
        m_logger = logger;
    }

    public int handleLink(
        String urlStr,
        String script,
        String line,
        int count,
        String scriptDir,
        BufferedReader reader,
        String outputDir,
        PrintStream print)
        throws IOException
    {
        StringTokenizer st = new StringTokenizer(script, ",");
        st.nextToken();
        String id = st.nextToken(); // save name
        String subFile = st.nextToken(); // sub script
        int orderOfLink = 0;
        if (st.hasMoreTokens() == true)
        {
            orderOfLink = new Integer(st.nextToken()).intValue();
        }

        while (line != null)
        {
            if (line.toLowerCase().indexOf("<a") > -1)
            {
                // 
                if (orderOfLink > 1)
                {
                    for (int i = 1; i < orderOfLink; i++)
                    {
                        int indexOfLinkStart = line.toLowerCase().indexOf("<a");
                        int indexOfLinkEnd = line.toLowerCase().indexOf("</a>");
                        line = line.substring(indexOfLinkEnd + 4);
                    }
                }

                // debug
                m_logger.debug("before select=" + line);

                // href
                int is = line.toLowerCase().indexOf("href") + 5;
                int ie =
                    line.toLowerCase().indexOf(
                        ">",
                        line.toLowerCase().indexOf("href") + 5);

                // remove " or '
                String href =
                    HtmlUtil.removeStr(line.substring(is, ie).trim(), "\"");
                href = HtmlUtil.removeStr(href, "'");
                if (href.indexOf(" ") > -1)
                {
                    href = href.substring(0, href.indexOf(" "));
                }
                m_logger.debug("href=" + href);

                // link text
                String selectStr = HtmlUtil.removeTagsAll(line).trim();
                if (selectStr.equals("") == false)
                {
                    //m_logger.debug("select=" + selectStr);
                    print.println("<a href=" + id + ">" + selectStr + "</a>");
                }

                String link;
                // ex) /test/test.html
                if (href.startsWith("/") == true)
                {
                    link = "http://" + (new URL(urlStr).getHost()) + href;
                }
                // ex) http://newhost/test/test.html
                else if (href.startsWith("http://") == true)
                {
                    link = href;
                }
                // ex) test/test.html
                else
                {
                    // ex) url =http://testserver/test.cgi?c=d
                    //     href=test.cgi?a=b

                    // get latest path
                    // ex) http://test/path1/path2/file => /path1/path2/
                    URL tempUrl = new URL(urlStr);
                    String tempPath = (tempUrl.getPath());
                    int oldIndex = -1, newIndex = -1;
                    if (tempPath.indexOf("/") > -1)
                    {
                        // next / found
                        oldIndex = tempPath.indexOf("/") + 1;
                        while (true)
                        {
                            newIndex = tempPath.indexOf("/", oldIndex);
                            if (newIndex == -1)
                            {
                                break;
                            }
                            else
                            {
                                oldIndex = newIndex + 1;
                            }
                        }
                    }

                    link =
                        "http://"
                            + tempUrl.getHost()
                            + tempPath.substring(0, oldIndex)
                            + href;

                    // old way
                    //link = urlStr + "/" + href;
                }

                readSite(link, scriptDir, subFile, outputDir, id);
            }
            if (line.toLowerCase().indexOf("</a>") > -1)
            {
                m_logger.debug("=====break======");
                break;
            }
            line = reader.readLine();
            count++;
        }

        return count;
    }

    public void handlePrint(String script, PrintStream print)
    {
        int i = script.indexOf(",");
        print.println(script.substring(i + 1));
    }

    public void handleTime(String script, PrintStream print)
    {
        Date date = new Date();
        SimpleDateFormat format = null;
        int i = script.indexOf(",");
        if (i > -1)
        {
            format = new SimpleDateFormat(script.substring(i + 1));
        }
        else
        {
            format = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        }
        print.println(format.format(date));
    }

    public int handleText(
        String script,
        String line,
        int count,
        BufferedReader reader,
        PrintStream print)
        throws IOException
    {
        StringTokenizer st = new StringTokenizer(script, ",");
        st.nextToken();
        String startTag = st.nextToken();
        String endTag = null;
        int mode = 0;
        if (st.hasMoreTokens() == true)
        {
            endTag = st.nextToken();
        }
        else
        {
            endTag = startTag;
            mode = 1;
        }
        while (line != null)
        {
            if (line.indexOf(startTag) > -1 || mode == 1)
            {
                //m_logger.debug("before select=" + line); // temp

                // startTag  Ѵ.
                if (mode == 0)
                {
                    line = line.substring(line.indexOf(startTag));
                }

                //   tag  Ѵ.
                String selectStr = HtmlUtil.removeTags(line);
                if (selectStr.equals("") == false)
                {
                    //m_logger.debug("select=" + selectStr);
                    print.println(selectStr);
                }
                mode = 1;
            }
            if (line.indexOf(endTag) > -1)
            {
                //m_logger.debug("=====break======"); // temp
                break;
            }
            line = reader.readLine();
            count++;
        }

        return count;
    }

    public int handleSkip(
        String script,
        String line,
        int count,
        BufferedReader reader)
        throws IOException, NullPointerException
    {
        StringTokenizer st = new StringTokenizer(script, ",");
        st.nextToken();
        //int seek = new Integer(st.nextToken()).intValue();
        String cmd = st.nextToken();
        //if (seek != 0)
        m_logger.debug("handleSkip: initial line=" + line);
        m_logger.debug("handleSkip: reader=" + reader);
        if (cmd.equals("curr") == true)
        {
            int lineNumber = new Integer(st.nextToken()).intValue();
            for (int i = 0; i < lineNumber; i++)
            {
                line = reader.readLine();
                count++;
            }
        }
        else if (cmd.equals("start") == true)
        {
            int lineNumber = new Integer(st.nextToken()).intValue();
            while (count < lineNumber)
            {
                line = reader.readLine();
                count++;
            }
        }
        else if (cmd.equals("until") == true)
        {
            String str = st.nextToken();
            m_logger.debug("until str=" + str);
            while (true)
            {
                m_logger.debug("until line=[" + line + "]");
                if (line.indexOf(str) > -1)
                {
                    break;
                }
                line = reader.readLine();
                m_logger.debug("until line=[" + line + "]");
                m_logger.debug("count=[" + count + "]");
                count++;
                if (line == null)
                {
                    m_logger.error("cannot found until string[" + str + "]");
                    break;
                }
            }
        }
        else
        {
            m_logger.error("unknown command: " + cmd);
        }

        return count;
    }

    public void handleRun(
        String script,
        int count,
        String scriptDir,
        BufferedReader reader,
        String outputDir,
        PrintStream print)
        throws IOException
    {
        // parse script
        StringTokenizer st = new StringTokenizer(script, ",");
        st.nextToken(); // run, ... , .
        String siteFilename = st.nextToken();
        String linkName = null;
        if (st.hasMoreTokens())
        {
            linkName = st.nextToken();
        }

        // read site file
        SiteFile siteFile = new SiteFile();
        siteFile.loadFromFile(scriptDir + "/" + siteFilename);

        // link
        if (linkName != null)
        {
            print.println(
                "<a href="
                    + siteFile.getOutputFilename()
                    + ">"
                    + linkName
                    + "</a>");
        }
        else
        {
            print.println(
                "<a href="
                    + siteFile.getOutputFilename()
                    + ">"
                    + siteFile.getDescription()
                    + "</a>");
        }

        // run other site file
        startScrapping(scriptDir, siteFilename, outputDir);
    }

    public void readSite(
        String urlStr,
        String scriptDir,
        String scriptFilename,
        String outputDir,
        String outputFilename)
        throws IOException
    {
        m_logger.debug("========< START >==========================");
        m_logger.info("read url=[" + urlStr + "]");
        m_logger.info("read file=[" + scriptFilename + "]");

        PrintStream print = null;
        try
        {
            print =
                new PrintStream(
                    new FileOutputStream(
                        new File(outputDir + "/" + outputFilename)));
        }
        catch (IOException e)
        {
            m_logger.error(
                "can't open output file["
                    + outputDir
                    + "/"
                    + outputFilename
                    + "]");
            e.printStackTrace();
            //            throw e;
        }

        BufferedReader readerScript = null;
        try
        {
            readerScript =
                new BufferedReader(
                    new InputStreamReader(
                        new FileInputStream(
                            new File(scriptDir + "/" + scriptFilename))));
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }

        URL url = null;
        try
        {
            url = new URL(urlStr);
        }
        catch (MalformedURLException e)
        {
            e.printStackTrace();
        }

        InputStream ins = null;
        BufferedReader reader = null;
        String script = null;
        String line = null;
        try
        {
            ins = url.openStream();
            reader = new BufferedReader(new InputStreamReader(ins));

            // get filename
            script = readerScript.readLine();
            m_logger.info("version=" + script);

            // get commands
            script = readerScript.readLine();
            m_logger.info("site title=" + script);

        }
        catch (FileNotFoundException e)
        {
            m_logger.error("url not found. url=[" + url + "]");
            e.printStackTrace();
            throw e;
        }
        catch (IOException e)
        {
            e.printStackTrace();
            throw e;
        }

        try
        {
            // start site reading
            script = readerScript.readLine();
            line = reader.readLine();

            m_logger.debug("readSite: first line=" + line);
            if (line == null)
            {
                m_logger.debug("readSite: first line is NULL");
            }
            else if (line.equals("") == true)
            {
                m_logger.debug("readSite: first line is empty");
            }
            int count = 1;
            while (script != null)
            {
                m_logger.debug("script=" + script); // temp
                m_logger.debug("count=" + count); // temp

                // skip command
                if (script.startsWith("skip") == true)
                {
                    try
                    {
                        count = handleSkip(script, line, count, reader);
                    }
                    catch (IOException e)
                    {
                        e.printStackTrace();
                    }
                }
                // link command
                else if (script.startsWith("link") == true)
                {
                    try
                    {
                        count =
                            handleLink(
                                urlStr,
                                script,
                                line,
                                count,
                                scriptDir,
                                reader,
                                outputDir,
                                print);
                    }
                    catch (IOException e)
                    {
                        e.printStackTrace();
                    }
                }
                // text command
                else if (script.startsWith("text") == true)
                {
                    try
                    {
                        count = handleText(script, line, count, reader, print);
                    }
                    catch (IOException e)
                    {
                        e.printStackTrace();
                    }
                }
                // print command
                else if (script.startsWith("print") == true)
                {
                    handlePrint(script, print);
                }
                // run command 
                // since 1.4
                else if (script.startsWith("run") == true)
                {
                    try
                    {
                        handleRun(
                            script,
                            count,
                            scriptDir,
                            reader,
                            outputDir,
                            print);

                    }
                    catch (IOException e)
                    {
                        e.printStackTrace();
                    }
                }
                // time command
                // since 1.4
                else if (script.startsWith("time") == true)
                {
                    handleTime(script, print);
                }
                // remark
                else if (
                    script.trim().equals("") == true
                        || script.startsWith("#") == true)
                {
                    // do nothing
                }
                else
                {
                    System.err.println("unknown command: " + script);
                }
                try
                {
                    script = readerScript.readLine();
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }

            // tail
            print.println("<p>" + TAIL_MESSAGE + "</p>");

            // close
            if (reader != null)
            {
                reader.close();
            }
            if (readerScript != null)
            {
                readerScript.close();
            }
        }
        catch (NullPointerException e)
        {
            m_logger.error("NullPointerException");
            e.printStackTrace();
        }
        catch (IOException e)
        {
            m_logger.error("can't read.");
            e.printStackTrace();
        }
        print.close();
        m_logger.debug("------------< END >------------");
    }

    public void startScrapping(
        String scriptDir,
        String siteFilename,
        String outputDir)
        throws IOException
    {
        m_logger.info("start scrapping...");

        // site file
        SiteFile siteFile = new SiteFile();
        siteFile.loadFromFile(scriptDir + "/" + siteFilename);
        m_logger.debug("Version=" + siteFile.getVersion());
        m_logger.debug("Site Description=" + siteFile.getDescription());

        // reading site (real scrapping)
        readSite(
            siteFile.getUrl(),
            scriptDir,
            siteFile.getScriptFilename(),
            outputDir,
            siteFile.getOutputFilename());

        m_logger.info("scrapping finished.");
    }

	public String getEngineVersion()
	{
		return ENGINE_VERSION;
	}
	
	public String getScriptVersion()
	{
		return SCRIPT_VERSION;
	}

    public static void main(String[] args)
    {
        if (args.length == 3)
        {
            Scraper app = new Scraper();
            try
            {
                app.startScrapping(args[0], args[1], args[2]);
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    }
}
