package yeoupooh.scraper;

import java.io.File;
import java.io.IOException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * @author pooh
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class ScraperHelper
{
    private String m_scriptDir = null;
    private String m_outputDir = null;
    private String m_mainSite = null;
    private Scraper m_scraper = null;
    private ILogger m_logger = null;

    public ScraperHelper()
    {
    	m_logger = new DefaultLogger();
        m_scraper = new Scraper();
        try
        {
            ResourceBundle bundle = ResourceBundle.getBundle("scraper");
            m_scriptDir = bundle.getString("script.dir");
            m_outputDir = bundle.getString("output.dir");
            m_mainSite = bundle.getString("main.site");
        }
        catch (MissingResourceException e)
        {
        	m_logger.error("resource bundle not found.");
            m_scriptDir = "scripts";
            m_outputDir = "output";
            m_mainSite = "main.site";
        }
    }

	public void setLogger(ILogger logger)
	{
		m_logger = logger;
	}

	public Scraper getScraper()
	{
		return m_scraper;
	}

    public void setScraperLogger(ILogger logger)
    {
        m_scraper.setLogger(logger);
    }


    public void setScriptDir(String dir)
    {
        m_scriptDir = dir;
    }

    public void setOutputDir(String dir)
    {
        m_outputDir = dir;
    }

    public void scrapSite(String siteFilename) throws IOException
    {
        m_scraper.startScrapping(m_scriptDir, siteFilename, m_outputDir);
    }

    public void scrapAllSite() throws IOException
    {
        String[] siteFilenames = getSiteFileList();
        for (int i = 0; i < siteFilenames.length; i++)
        {
            String siteFilename = siteFilenames[i];
            scrapSite(siteFilename);
        }
    }

    public void scrapMainSite() throws IOException
    {
        scrapSite(m_mainSite);
    }

    public String[] getSiteFileList()
    {
        File file = new File(m_scriptDir);
        return file.list(new SiteFileFilter());
    }

    /**
     * Returns the mainScript.
     * @return String
     */
    public String getMainSite()
    {
        return m_mainSite;
    }

    /**
     * Sets the mainScript.
     * @param mainScript The mainScript to set
     */
    public void setMainSite(String mainScript)
    {
        m_mainSite = mainScript;
    }

    public static void main(String[] args)
    {
        ScraperHelper app = new ScraperHelper();
        app.setScriptDir("scripts");
        app.setOutputDir("output");
        try
        {
            app.scrapAllSite();

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

}
